@@ -1,17 +1,18 @@
 # -*- coding: utf-8 -*-
-import contextlib
+import logging
 import math
 
 import pandas
 from pytdx.exhq import TdxExHq_API
 from pytdx.hq import TdxHq_API
+from tenacity import retry, stop_after_attempt, retry_if_result, retry_if_exception_type, before_log
 from tqdm import tqdm
 from unipath import Path
 
 from mootdx import config, server
 from mootdx.consts import MARKET_SH
 from mootdx.logger import log
-from mootdx.utils import (get_config_path, get_stock_market, get_stock_markets, to_data)
+from mootdx.utils import get_config_path, get_stock_market, get_stock_markets, to_data
 
 
 class Quotes(object):
@@ -39,20 +40,7 @@ class BaseQuotes(object):
 
         self.timeout = timeout
 
-        config_ = get_config_path('config.json')
-
-        if bestip or not Path(config_).exists():
-            server.bestip()
-
-    @contextlib.contextmanager
-    def connect(self):
-        if self.closed:
-            log.debug('服务器连接已断开，正进行重新连接...')
-            self.reconnect()
-
-        yield
-
-        self.close()
+        Path(get_config_path('config.json')).exists() or server.bestip()
 
     def __del__(self):
         log.debug('__del__')
@@ -62,6 +50,7 @@ class BaseQuotes(object):
         if self.closed:
             log.debug('服务器连接已断开，正进行重新连接...')
             self.client.connect(time_out=self.timeout, *self.bestip)
+            # self.client.connect(*self.bestip)
 
     def close(self):
         log.debug('close')
@@ -69,36 +58,56 @@ class BaseQuotes(object):
 
     @property
     def closed(self):
-        if hasattr(self.client.client, '_closed') and not getattr(self.client.client, '_closed'):
-            return False
+        if not hasattr(self.client.client, '_closed') or getattr(self.client.client, '_closed'):
+            return True
 
-        return True
+        return False
+
+
+instance: BaseQuotes = None
+
+
+def is_empty(value):
+    """Return True if value is False"""
+
+    status = bool(value.empty) if hasattr(value, 'empty') else bool(value) is False
+
+    print('is_empty is ', status)
+
+    if instance and status:
+        instance.client.connect(time_out=instance.timeout, *instance.bestip)
+
+    return status
 
 
 class StdQuotes(BaseQuotes):
     """
     股票市场实时行情
     """
-    bestip = ('47.103.48.45', 7709)
+    # bestip = ('47.103.48.45', 7709)
 
     def __init__(self, bestip=False, timeout=15, **kwargs):
         super(StdQuotes, self).__init__(bestip=bestip, timeout=timeout, **kwargs)
 
         try:
-            default = config.get('SERVER').get('HQ')[0]
+            config.get('SERVER').get('HQ')[0]
         except ValueError:
             server.bestip()
         finally:
             default = config.get('SERVER').get('HQ')[0]
             self.bestip = config.get('BESTIP').get('HQ', default)
 
-        self.client = TdxHq_API(**kwargs)
+        self.client = TdxHq_API(raise_exception=False, **kwargs)
         self.client.connect(*self.bestip)
 
+        global instance
+        instance = self
+
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def traffic(self):
-        self.reconnect()
         return self.client.get_traffic_stats()
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def quotes(self, symbol=None):
         """
         获取实时日行情数据
@@ -113,13 +122,12 @@ class StdQuotes(BaseQuotes):
         if type(symbol) is str:
             symbol = [symbol]
 
-        self.reconnect()
-
         symbol = get_stock_markets(symbol)
         result = self.client.get_security_quotes(symbol)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def bars(self, symbol='000001', frequency=9, start=0, offset=100, *args, **kwargs):
         """
         获取实时日K线数据
@@ -130,7 +138,6 @@ class StdQuotes(BaseQuotes):
         :param offset: 每次获取条数
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         offset = 800 if int(offset) >= 800 else offset
@@ -138,6 +145,7 @@ class StdQuotes(BaseQuotes):
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def stock_count(self, market=MARKET_SH, *args, **kwargs):
         """
         获取市场股票数量
@@ -145,12 +153,12 @@ class StdQuotes(BaseQuotes):
         :param market: 股票市场代码 sh 上海， sz 深圳
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         result = self.client.get_security_count(market=market)
 
         return result
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def stocks(self, market=MARKET_SH, *args, **kwargs):
         """
         获取股票列表
@@ -158,7 +166,6 @@ class StdQuotes(BaseQuotes):
         :param market: 股票市场
         :return:
         """
-        self.reconnect()
 
         counts = self.client.get_security_count(market=market)
         stocks = None
@@ -169,6 +176,7 @@ class StdQuotes(BaseQuotes):
 
         return stocks
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def index_bars(self, symbol='000001', frequency=9, start=0, offset=100, *args, **kwargs):
         """
         获取指数k线
@@ -179,13 +187,13 @@ class StdQuotes(BaseQuotes):
         :param offset:
         :return:
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         result = self.client.get_index_bars(frequency=frequency, market=market, code=symbol, start=start, count=offset)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def minute(self, symbol=''):
         """
         获取实时分时数据
@@ -193,13 +201,13 @@ class StdQuotes(BaseQuotes):
         :param symbol: 股票代码
         :return: pd.DataFrame
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         result = self.client.get_minute_time_data(market=market, code=symbol)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def minutes(self, symbol='', date='20191023', *args, **kwargs):
         """
         分时历史数据
@@ -208,13 +216,13 @@ class StdQuotes(BaseQuotes):
         :param date:
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         result = self.client.get_history_minute_time_data(market=market, code=symbol, date=date)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def transaction(self, symbol='', start=0, offset=10, *args, **kwargs):
         """
         查询分笔成交
@@ -224,13 +232,13 @@ class StdQuotes(BaseQuotes):
         :param offset: 结束位置
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         result = self.client.get_transaction_data(int(market), symbol, start, offset)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def transactions(self, symbol='', start=0, offset=10, date='20170209'):
         """
         查询历史分笔成交
@@ -242,13 +250,13 @@ class StdQuotes(BaseQuotes):
         :param date: 日期
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         market = get_stock_market(symbol, string=False)
         result = self.client.get_history_transaction_data(market=market, code=symbol, start=start, count=offset, date=int(date))
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def F10C(self, symbol=''):
         """
         查询公司信息目录
@@ -256,13 +264,12 @@ class StdQuotes(BaseQuotes):
         :param symbol: 股票代码
         :return: pd.dataFrame or None
         """
-        self.reconnect()
-
         market = int(get_stock_market(symbol))
         result = self.client.get_company_info_category(market, symbol)
 
         return result
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def F10(self, symbol='', name=''):
         """
         读取公司信息详情
@@ -271,13 +278,14 @@ class StdQuotes(BaseQuotes):
         :param symbol: 股票代码
         :return: pd.dataFrame or None
         """
-        self.reconnect()
-
         result = {}
         market = int(get_stock_market(symbol, string=False))
 
         frequency = self.client.get_company_info_category(market, symbol)
 
+        if not frequency:
+            return None
+
         if name:
             for x in frequency:
                 if x['name'] == name:
@@ -289,6 +297,7 @@ class StdQuotes(BaseQuotes):
 
         return result
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def xdxr(self, symbol=''):
         """
         读取除权除息信息
@@ -296,13 +305,13 @@ class StdQuotes(BaseQuotes):
         :param symbol: 股票代码
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         result = self.client.get_xdxr_info(int(market), symbol)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def finance(self, symbol='000001'):
         """
         读取财务信息
@@ -310,13 +319,13 @@ class StdQuotes(BaseQuotes):
         :param symbol: 股票代码
         :return:
         """
-        self.reconnect()
 
         market = get_stock_market(symbol)
         result = self.client.get_finance_info(market=market, code=symbol)
 
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def k(self, symbol='', begin=None, end=None):
         """
         读取k线信息
@@ -326,11 +335,11 @@ class StdQuotes(BaseQuotes):
         :param end: 截止日期
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         result = self.client.get_k_data(symbol, begin, end)
         return result
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def index(self, symbol='000001', market=MARKET_SH, frequency=9, start=1, offset=2):
         """
         获取指数k线
@@ -357,11 +366,11 @@ class StdQuotes(BaseQuotes):
         :param offset: 每次获取条数
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         result = self.client.get_index_bars(int(frequency), int(market), str(symbol), int(start), int(offset))
         return to_data(result)
 
+    @retry(stop=stop_after_attempt(3), retry=(retry_if_exception_type() | retry_if_result(is_empty)))
     def block(self, tofile="block.dat"):
         """
         获取证券板块信息
@@ -369,7 +378,6 @@ class StdQuotes(BaseQuotes):
         :param tofile:
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         result = self.client.get_and_parse_block_info(tofile)
         return to_data(result)
@@ -386,7 +394,7 @@ class ExtQuotes(BaseQuotes):
         super(ExtQuotes, self).__init__(bestip=bestip, timeout=timeout, **kwargs)
 
         try:
-            default = config.get('SERVER').get('EX')[0]
+            config.get('SERVER').get('EX')[0]
         except ValueError:
             server.bestip()
         finally:
@@ -414,7 +422,6 @@ class ExtQuotes(BaseQuotes):
 
         :return: pd.dataFrame or None
         """
-        self.reconnect()
 
         result = self.client.get_markets()
         return to_data(result)
@@ -427,7 +434,6 @@ class ExtQuotes(BaseQuotes):
         :param start:
         :return:
         """
-        self.reconnect()
 
         result = self.client.get_instrument_info(start=start, count=offset)
         return to_data(result)
@@ -438,7 +444,6 @@ class ExtQuotes(BaseQuotes):
 
         :return:
         """
-        self.reconnect()
 
         result = self.client.get_instrument_count()
 
@@ -450,7 +455,6 @@ class ExtQuotes(BaseQuotes):
 
         :return:
         """
-        self.reconnect()
 
         result = []
 
@@ -470,7 +474,6 @@ class ExtQuotes(BaseQuotes):
         :param symbol: 证券代码
         :return:
         """
-        self.reconnect()
 
         market, symbol = self.validate(market, symbol)
         result = self.client.get_instrument_quote(market, symbol)
@@ -485,7 +488,6 @@ class ExtQuotes(BaseQuotes):
         :param symbol: 证券代码
         :return:
         """
-        self.reconnect()
 
         market, symbol = self.validate(market, symbol)
         result = self.client.get_minute_time_data(market, symbol)
@@ -501,7 +503,6 @@ class ExtQuotes(BaseQuotes):
         :param date:
         :return:
         """
-        self.reconnect()
 
         market, symbol = self.validate(market, symbol)
         result = self.client.get_history_minute_time_data(market, symbol, date)
@@ -520,7 +521,6 @@ class ExtQuotes(BaseQuotes):
         :param offset: 数量
         :return:
         """
-        self.reconnect()
 
         market, symbol = self.validate(market, symbol)
         result = self.client.get_instrument_bars(category=frequency, market=market, code=symbol, start=start, count=offset)
@@ -537,7 +537,6 @@ class ExtQuotes(BaseQuotes):
         :param offset:
         :return:
         """
-        self.reconnect()
 
         market, symbol = self.validate(market, symbol)
         result = self.client.get_transaction_data(market=market, code=symbol, start=start, count=offset)
@@ -555,7 +554,6 @@ class ExtQuotes(BaseQuotes):
         :param offset:
         :return:
         """
-        self.reconnect()
 
         market, symbol = self.validate(market, symbol)
         result = self.client.get_history_transaction_data(market=market, code=symbol, date=int(date), start=start, count=offset)
