@@ -1,18 +1,14 @@
 # -*- coding: utf-8 -*-
 import json
-import logging
 import socket
+import sys
 import time
 
-from prettytable import PrettyTable
-
 from mootdx import config
 from mootdx.consts import CONFIG, EX_HOSTS, GP_HOSTS, HQ_HOSTS
 from mootdx.logger import log
 from mootdx.utils import get_config_path
 
-result = []
-
 hosts = {
     'HQ': [{
         'addr': hs[1],
@@ -37,19 +33,15 @@ hosts = {
 
 def Server(index=None, limit=5, console=False, verbose=False):
     if verbose:
-        logging.basicConfig(level='DEBUG')
+        log.add(sys.stdout, level='DEBUG')
 
     _hosts = hosts[index]
+
     server = []
 
-    while True:
-        if len(_hosts) == 0:
-            break
+    while len(_hosts) > 0:
 
-        try:
-            proxy = _hosts.pop()
-        except ValueError:
-            return
+        proxy = _hosts.pop()
 
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.settimeout(5)
@@ -63,15 +55,16 @@ def Server(index=None, limit=5, console=False, verbose=False):
             proxy['time'] = (time.perf_counter() - start) * 1000
 
             server.append(proxy)
-            log.info("{}:{} 验证通过，响应时间：{:5.2f} ms.".format(proxy.get('addr'), proxy.get('port'), proxy.get('time')))
+
+            log.debug("{}:{} 验证通过，响应时间：{:5.2f} ms.".format(proxy.get('addr'), proxy.get('port'), proxy.get('time')))
         except Exception as e:
-            log.error(e)
-            log.warning("{},{} 验证失败.".format(proxy.get('addr'), proxy.get('port')))
+            log.debug("{},{} 验证失败.".format(proxy.get('addr'), proxy.get('port')))
 
     # 结果按响应时间从小到大排序
-    server.sort(key=lambda item: item['time'])
 
     if console:
+        from prettytable import PrettyTable
+        server.sort(key=lambda item: item['time'])
         print("[√] 最优服务器:")
 
         t = PrettyTable(["Name", "Addr", "Port", "Time"])
@@ -82,25 +75,21 @@ def Server(index=None, limit=5, console=False, verbose=False):
         t.padding_width = 1
 
         for host in server[:int(limit)]:
-            t.add_row([
-                host['site'], host['addr'], host['port'],
-                '{:5.2f} ms'.format(host['time'])
-            ])
+            t.add_row([host['site'], host['addr'], host['port'], '{:5.2f} ms'.format(host['time'])])
 
         print(t)
 
     return [(item['addr'], item['port']) for item in server]
 
 
-def bestip():
+def bestip(verbose=False, console=False, limit=5) -> None:
     config_ = get_config_path('config.json')
     default = dict(CONFIG)
 
     for index in ['HQ', 'EX', 'GP']:
-        result = Server(index=index)
-
-        if result:
-            default['BESTIP'][index] = result[0]
+        data = Server(index=index, limit=limit, console=console, verbose=verbose)
+        if data:
+            default['BESTIP'][index] = data[0]
 
     json.dump(default, open(config_, 'w'), indent=2)
     config.setup()
